#
# makefile related checks
#

#use strict;
use LWP;
use Data::Dumper;

########################################  FUNCTIONS BELOW THIS POINT

sub propermansect {
	my (%vars) = @_;
	foreach my $sectname (split(/\s+/, <<EOF)) {
MAN1 MAN2 MAN3 MAN4 MAN5 MAN6 MAN7 MAN8 MAN9 MANL MANN
EOF
		$sectname =~ m/(.)$/;
		my $sect = $1;
		foreach my $page (split(/\s+/, $vars{$sectname})) {
			$page =~ m/(.+)\.(.)(\..+)?/;
			my $newsect = $2;
			if ($sect != $newsect) {
				&perror("WARN: Makefile: $page listed in $sectname");
				my $newprevlst = $vars{"MAN$sect"};
				$newprevlst =~ s/$page//;
				my $newnextlst = $vars{"MAN$newsect"};
				$newnextlst =~ s/$/ $page/;
				$newnextlst =~ s/^ //;
				&createpatch("Makefile", "^MAN$sect", "MAN$sect=\t$vars{'MAN' . $sect}", "MAN$sect=\t$newprevlst\nMAN$newsect=\t$newnextlst");
			}
		}
	}
}

# this function's params seem kinda ackward.  Oh well.
sub nouservar {
	my ($file, $varname, $varvalue) = @_;
	
	if ($varvalue ne "") {
		&perror("FATAL: $file: use of $varname is discouraged. ".
			"it is intended to be a user variable.");
	} else {
		&verbose("$varname not found in $file");
	}
}

#MICHAEL: should probably use make vars and not parse Makefile for this
sub noportdocs {
	my ($file, $whole, $prefix) = @_;

#MICHAEL: not tested
	if ($whole =~ m/\${PREFIX}\/share\// && $whole !~ m/^\.if \(?\!defined\(NOPORTDOCS\)/m) {
		&perror("WARN: use \".if !defined(NOPORTDOCS)\" to wrap installation of files into $prefix/share/doc.");
	} else {
		&verbose("any files being installed into $prefix are fine");
	}
}

#MICHAEL: not tested, just "ported" from portlint2 so not even checked
#         for validity
sub is_predefined {
	my($url, %predefined) = @_;
	my($site);
	my($subdir);
	if ($site = (grep($url =~ $_, keys %predefined))[0]) {
		$url =~ /$site/;
		$subdir = $';
		$subdir =~ s/\/$//;
		&perror("WARN: how about using ".
			"\${MASTER_SITE_$predefined{$site}} with ".
			"\"MASTER_SITE_SUBDIR=$subdir\", instead of \"$url\?");
	}
}

########################################  FUNCTION REGISTRATION

@func_reg = (
#{
#  function => \&validcategories,
#  name => "validcategories",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&propermansect,
#  name => "propermansect",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&nouservar,
#  name => "nouservar",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&noportdocs,
#  name => "noportdocs",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&is_predefined,
#  name => "is_predefined",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&sectionsplit,
#  name => "sectionsplit",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&sect0lines,
#  name => "sect0lines",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&commentsonly,
#  name => "commentsonly",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&commentsstrip,
#  name => "commentsstrip",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&maintainerexists,
#  name => "maintainerexists",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&maintainervalidemail,
#  name => "maintainervalidemail",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&maintaineremailappend,
#  name => "maintaineremailappend",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&restrictedcheck,
#  name => "restrictedcheck",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&nocdromcheck,
#  name => "nocdromcheck",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&nopackagecheck,
#  name => "nopackagecheck",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&noconfigurecheck,
#  name => "noconfigurecheck",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
#{
#  function => \&nopatchcheck,
#  name => "nopatchcheck",
#  option => undef,
#  enabled => 1,
#  file => "Makefile"
#},
);

#print Data::Dumper->Dump([\@func_reg],['@func_reg']);
\@func_reg;
