#use strict;

###################################################
# Warning:  All work done beyond this point.  :-)
###################################################

#XXX

#
# pkg/ related stuff
#
sub descrwwwcheck {
	my ($file, $whole) = @_;

	if ($whole =~ m,http://,) {
		my $has_url = 0;
		my $has_www = 0;
		foreach my $line (grep($_ =~ "http://", split(/\n+/, $whole))) {
			$has_url = 1;
			$line =~ m,(http://[\w\-\.]+),;
			my $url = $1;
			&geturlstatus($url);
			if ($line =~ m,WWW:[ \t]+http://,) {
				$has_www = 1;
			}
		}

		if ($has_url && ! $has_www) {
			&perror("FATAL: $file: contains a URL but no WWW:");
		} else {
			&verbose("$file a URL is prefixed with WWW:");
		}
	}
}

sub commentstylecheck {
	my ($file, $whole) = @_;
	
	if ($whole !~ /^["0-9A-Z]/) {
		&perror("WARN: $file: Line should begin with a capital letter")
	}
	if ($whole =~ /\.$/) {
		&perror("WARN: $file: Line should end without a period.");
	}
}

#
# files/ related stuff
#

#
# patches/ related stuff
#

#
# scripts/ related stuff
#

#
# other checks
#

sub workdircheck {
	if (-e "work") {
		&perror("FATAL: be sure to 'make clean' before ".
			"submitting/committing.");
	} else {
		&verbose("work directory doesn't exist");
	}
}

sub warncrud {
	my ($dir) = @_;

	my @ls = ();
	my $problem = 0;

	opendir(D, $dir) || errx(1, "opendir .");
	@ls = readdir(D);
	closedir(D);
	foreach my $item (@ls) {
		next if ($item =~ /^(\.|\.\.|CVS)$/);
		warncrud("$dir/$item") if (-d "$dir/$item");
		if ($item =~ /~$/) {
			&perror("FATAL: $dir/$item: for safety, be sure to cleanup ".
				"editor backup files before submitting/committing.");
			$problem = 1;
		}
		if ($item =~ /\.rej$/ || $item =~ /\.orig$/) {
			&perror("FATAL: $dir/$item: for safety, be sure to cleanup ".
				"patch backup files before submitting/committing.");
			$problem = 1;
		}
		if (-z "$item") {
			&perror("FATAL: $item: empty file and should be removed.");
			$problem = 1;
		}
	}

	if (! $problem) {
		&verbose("no crud found in $dir");
	}
}


#
# general checking
#

sub loadfile {
	my ($file) = @_;
	my $linecnt = 0;
	my $maxlinelen = 0;
	my $len;
	my $whole = "";
	
	if (!open(F, "< $file")) {
		&perror("FATAL: unable to open $file");
		return ("", 0, 0);
	}
	while(<F>) {
		$whole .= $_;
		$linecnt++;
		$len = length(chomp($_));
		$maxlinelen = $len if ($len > $maxlinelen);
	}
	close(F);

	if ($whole =~ /[\033\200-\377]/) {
		&perror("WARN: $file: including iso-8859-1, or ".
			"other local characters.  Should be a plain ascii file");
	}

	return ($whole, $linecnt, $maxlinelen);
}

sub urlcheck {
	my ($url) = @_;
	if ($url !~ m#^\w+://#) {
		&perror("WARN: \"$url\" doesn't appear to be a URL to me.");
	}
	if ($url !~ m#/$#) {
		&perror("FATAL: URL \"$url\" should end with \"/\".");
	}
	if ($url =~ m#://[^/]*:/#) {
		&perror("FATAL: URL \"$url\" contains extra \":\".");
	}
	if ($url =~ m#(www.freebsd.org)/~.+/#i) {
		&perror("WARN: URL \"$url\", $1 should be people.FreeBSD.org");
	}
}

sub geturlstatus {
	my ($url) = @_;

	return if $checkedurls{"$url"} == 1;

	my $ua = new LWP::UserAgent;
	my $request = new HTTP::Request('GET', $url);
	my $response = $ua->request($request);

	my ($statuscode, $statusmessage) = split(/ /, $response->status_line, 2);

	if ($statuscode != 200) {
		&perror("WARN: URL \'$url\' returned error $statuscode, $statusmessage");
	}
	$checkedurls{"$url"} = 1;
}

sub maxlines {
	my ($file, $count, $max) = @_;

	if ($count > $max) {
		&perror("WARN: $file exceeds $max lines, make it shorter if possible. (currently $count lines)");
	} else {
		&verbose("$file has $count line(s).");
	}
}

sub maxchars {
	my ($file, $count, $max) = @_;

	if ($count > $max) {
		&perror("WARN: $file: includes lines that exceed $max characters.");
	} else {
		&verbose("$file has at most $max characters per line.");
	}
}

sub whitespaceatend {
	my ($file, $whole) = @_;
	
	my $problem = 0;
	my $i = 1;
	foreach my $line (split(/\n/, $whole)) {
		if ($line =~ /[ \t]$/) {
			&perror("WARN: $file $i: whitespace before end of line");
			$problem = 1;
		}
		$i++;
	}

	if (! $problem) {
		&verbose("$file has no whitespace before the end of a line");
	}
}

#MICHAEL: maybe include the line number this is found on
sub blanklines {
	my ($file, $whole, $contblank) = @_;
	
	my $blanks = "\n" x ($contblank + 2);

	if ($whole =~ /$blanks/) {
		&perror("FATAL: $file: contiguous blank lines (> $contblank lines) found");
	} else {
		&verbose("$file has no contiguous blank lines");
	}
}

sub lastline {
	my ($file, $whole) = @_;

	my $problems = 0;
	
	if ($whole !~ /\n$/) {
		&perror("FATAL: $file: last line needs to be terminated by a \\n");
		$problems=1;
	}
	if ($whole =~ /\n([ \t]*\n)+$/) {
		&perror("WARN: $file: last line seems to be blank.");
		$problems=1;
	}
	if (! $problems) {
		&verbose("last line of $file looks good to me");
	}
}


#
# portlint general functions
#
sub contains {
	my ($search, @list) = @_;

	for (@list) {
		return 1 if m/^\Q$search\E$/;
	}
	return 0;
}

sub checkearlier {
	my ($file, $whole, @varnames) = @_;

	my $i;
	foreach my $i (@varnames) {
		if ($whole =~ /\n$i[?+]?=/) {
			&perror("WARN: \"$i\" has to appear earlier in $file.");
		}
	}
}

sub getmakevars {
	my (@varlist) = @_;
	my %hash;
	my $cmd = "make -DPORTLINTING $opts{makeenv} MASTER_SITE_BACKUP=''";
	my $i = 0;

	for (@varlist) {
		$cmd .= " -V $_";
	}

	for (split(/\n/, `$cmd`)) {
		$hash{$varlist[$i]} = $_;
		if ((defined($opts{'display-makevars'}) && $opts{'display-makevars'} != 0 && $opts{'verbose'}) || ($opts{'verbose'} && !defined($opts{'display-makevars'})) || $opts{'display-makevars'} ) {
			print "makevar: $varlist[$i] = $_\n";
		}
		$i++;
	}

	return %hash;
}

sub createpatch {
	my ($file, $pattern, $old, $new) = @_;

	return if ($opts{'patch'} ne 1);
	my $patchfile = "portlint.patch";
	$patchfile = $opts{'patchfile'} if $opts{'patchfile'} ne "";
	$patchfile = "$patchfile-$patchcount";

#MICHAEL: not sure if patch creation is correct
	my ($oldline, $oldlen, $newline, $newlen);
	my $oldout;
	my $newout;

	# stupid hack that I don't believe is the right way to do this
	my $beginning = 0;
	$pattern =~ s/^(\^)//;
	$beginning = 1 if $1 ne "";

	open(F, "$file") || die "$!";
	while(<F>) {
		if ($beginning) {
			last if (m/(^$pattern)/);
		} else {
			last if (m/($pattern)/);
		}
	}
	$oldline = $.;
	$newline = $.;
	if ($1 eq "") {
		warnx("unable to match pattern \'$pattern\' in file $file");
		return;
	}
	close(F);
	foreach my $line (split(/\n/, $old)) {
		$oldout .= "- $line\n";
		$oldlen++;
	}
	foreach my $line (split(/\n/, $new)) {
		$newout .= "+ $line\n";
		$newlen++;
	}

	errx(1, "\`$patchfile\' file already exists.  I can't handle this yet") if ( -e "$patchfile");
	open(DIFF, ">>$patchfile");
	print DIFF <<EOF;
--- $file
+++ $file
@@ -$oldline +$newline,$newlen @@
EOF
	print DIFF "$oldout$newout";
}

1;
